import { EventsKey } from '../events';
import Event from '../events/Event';
import ImageTile from '../ImageTile';
import { ObjectEvent } from '../Object';
import { ProjectionLike } from '../proj';
import { LoadFunction, UrlFunction } from '../Tile';
import WMTSTileGrid from '../tilegrid/WMTS';
import { AttributionLike } from './Source';
import { TileSourceEvent } from './Tile';
import TileImage from './TileImage';
import WMTSRequestEncoding from './WMTSRequestEncoding';

export interface Options {
    attributions?: AttributionLike | undefined;
    cacheSize?: number | undefined;
    crossOrigin?: string | undefined;
    tileGrid: WMTSTileGrid;
    projection?: ProjectionLike | undefined;
    reprojectionErrorThreshold?: number | undefined;
    requestEncoding?: WMTSRequestEncoding | string | undefined;
    layer: string;
    style: string;
    tileClass?: ImageTile | undefined;
    tilePixelRatio?: number | undefined;
    format?: string | undefined;
    version?: string | undefined;
    matrixSet: string;
    dimensions?: any;
    url?: string | undefined;
    tileLoadFunction?: LoadFunction | undefined;
    urls?: string[] | undefined;
    wrapX?: boolean | undefined;
    transition?: number | undefined;
}
export default class WMTS extends TileImage {
    constructor(options: Options);
    getDimensions(): any;
    getFormat(): string;
    getLayer(): string;
    getMatrixSet(): string;
    getRequestEncoding(): WMTSRequestEncoding;
    getStyle(): string;
    getVersion(): string;
    updateDimensions(dimensions: any): void;
    on(type: string | string[], listener: (p0: any) => void): EventsKey | EventsKey[];
    once(type: string | string[], listener: (p0: any) => void): EventsKey | EventsKey[];
    un(type: string | string[], listener: (p0: any) => void): void;
    on(type: 'change', listener: (evt: Event) => void): EventsKey;
    once(type: 'change', listener: (evt: Event) => void): EventsKey;
    un(type: 'change', listener: (evt: Event) => void): void;
    on(type: 'propertychange', listener: (evt: ObjectEvent) => void): EventsKey;
    once(type: 'propertychange', listener: (evt: ObjectEvent) => void): EventsKey;
    un(type: 'propertychange', listener: (evt: ObjectEvent) => void): void;
    on(type: 'tileloadend', listener: (evt: TileSourceEvent) => void): EventsKey;
    once(type: 'tileloadend', listener: (evt: TileSourceEvent) => void): EventsKey;
    un(type: 'tileloadend', listener: (evt: TileSourceEvent) => void): void;
    on(type: 'tileloaderror', listener: (evt: TileSourceEvent) => void): EventsKey;
    once(type: 'tileloaderror', listener: (evt: TileSourceEvent) => void): EventsKey;
    un(type: 'tileloaderror', listener: (evt: TileSourceEvent) => void): void;
    on(type: 'tileloadstart', listener: (evt: TileSourceEvent) => void): EventsKey;
    once(type: 'tileloadstart', listener: (evt: TileSourceEvent) => void): EventsKey;
    un(type: 'tileloadstart', listener: (evt: TileSourceEvent) => void): void;
}
export function optionsFromCapabilities(wmtsCap: any, config: any): Options;
